<?php
ini_set('display_startup_errors',1);
ini_set('display_errors',1);
error_reporting(-1);
error_reporting(E_ALL);
#-----------------------------------------------

# CREDIT - DO NOT REMOVE WITHOUT PERMISSION
# Original Author:       : Wim van der Kuil
# Highly modified parsing script for Davis Airlink sensors - by the beteljuice
# (data) json file should be regularly uploaded to webspace !!!

# Ver 1.1 - 15 Oct 2020

# Can provide pre-defined data body, 'key' table for various national standards, and (some) languages,
# and a 'widget' ("Nowcast" 2.5pm weighted value)
# default data content is configurable (below) 
# there are some php variables available so you can 'home brew' your own page content
#
# NB: Containing page should be UTF-8 else change ALL µ to &mu; (µ gives a better tail for some reason ?)
#
# ------------------------------------- SETTINGS
#

$useOwnJSON = "http://wareham-wx.ddns.net/v1/current_conditions"; // url of 'donor' json file

# these settings are related to displayed output
#------------------------  calculate AQHI (default)
$aqhi_type      = 'UK';         # base calculations on (US) = airnow.gov or on other country standards (BE) (eea*) (NL) (IE) (UK*) *beteljuice
$AQ_allowed        = array ('BE', 'eea', 'IE', 'NL', 'UK', 'US'); // options available / shown in picker *MUST* be at least $aqhi_type !!
#
$AQ_arr_all        = array('10','25','1'); # all sensors to display  #'1','25','10'
$AQ_time_line	= array('1m',	# avg. time-lines to display - NOT using '_last' (20s) value
						'1h',
						'3h',
						'12h',
						'24h'
						); // NB: 24h avg 2.5um is what the 'key' is calibrated to ! - 12h = "Nowcast"

$AQtemp_uom = "C"; 	// if used, $AQ_temp "C" or "F"

# ----some more    SETTINGS   ------    if used in template will be replaced by ....
$AQ_language       = 'en';                 	# $SITE["lang"]

$AQ_timezone       = 'Europe/London';    	# $SITE["tz"] 
$AQ_local_date_fmt  = 'd M Y H:i T';          # $SITE["timeFormat"] // ok beteljuice has put month short english, no look-up stuff for lang
$AQ_local_time_fmt  = 'H:i';                # $SITE["timeOnlyFormat"]
#

$debugOn        = true;
#$debugOn        = false;


// $is_current     = '25';                 # particle size for the 'widget' - now 'fixed' (beteljuice)
$current_style  = 'font-size: 16pt; -webkit-border-radius: 5px; border-radius: 5px; padding: 2px;';	# for the 'widget' beteljuice - added padding
#
#-----------------------------------------------
#               END OF USER SETTINGS
#-----------------------------------------------


#-----------------------------------------------
#  display source of script if requested so
#-----------------------------------------------
if (isset($_REQUEST['sce']) && strtolower($_REQUEST['sce']) == 'view' ) 
     {  $filenameReal = __FILE__;			
        $download_size = filesize($filenameReal);
        header('Pragma: public');
        header('Cache-Control: private');
        header('Cache-Control: no-cache, must-revalidate');
        header("Content-type: text/plain");
        header("Accept-Ranges: bytes");
        header("Content-Length: $download_size");
        header('Connection: close');
        readfile($filenameReal);
        exit;}
#-----------------------------------------------
#              check if needed functions are set
load_missing();
#-----------------------------------------------
$pageName	= 'betel_AQ_davis.php'; 
$pageVersion	= '1.00 30 Aug 2020'; 
$pageUpdated	= 'Release version '; 
#--------------------------------------- 
# 
# ---------------------------------- Houskeeping
$pageLoaded 	= basename(__FILE__); 
$string         = $pageVersion.' | '.$pageUpdated;
if ( $pageName <>  $pageLoaded)  {  $string .= ' => check script name: '.$pageName; } 
$SITE['wsModules'][$pageLoaded]	= $string;
ws_message ('<!-- loaded script: '.$pageLoaded.' '.$string.' -->');
# ---------------------------- check environment 

#
if (isset ($SITE['lang']) )             { $AQ_language =  $SITE['lang'];}
if (!isset ($lang) )                    { $lang =  $AQ_language;}
if (isset($_REQUEST['lang']) )          { $lang =  $_REQUEST['lang'];}
#
if (!isset ($SITE["tz"]))               { date_default_timezone_set($AQ_timezone);}
#
if (isset ($SITE["timeFormat"]) )       { $AQ_local_time_fmt  =  $SITE["timeFormat"];}

// for index picker if used (or php?AQtype= )
if(isset($_REQUEST['AQtype']) ) { $aqhi_type = $_REQUEST['AQtype'];}

#   
#-------------- check legends / threshold values
#

if (!isset ($aqhi_type) || !in_array($aqhi_type,$AQ_allowed)) 
     {  $aqhi_type      = 'US';}
#
ws_load_aq_langs ();
aqhi_load_arrays ();
#
if (isset($_REQUEST['debug']) )         { $wsDebug  = true;}
else                                    { $wsDebug  = $debugOn;}
#
# ------------------------------- set file names
$ws_msg_string  = false;
#
ws_message (ws_debug_times('module '.basename(__FILE__).' ('.__LINE__.'):'),true); 
#
ws_message('<!-- module '.basename(__FILE__).'('.__LINE__.'): file '.$useOwnJSON.' used, -->',true );
$url    = $useOwnJSON;
# get data file
$new_data = ws_makeRequest($url, false); 
// *** From here on Davis specific decoding for data table ....
// print_r($new_data);
$AQ_error = false;
$current_AQ_risk = '';

if ($new_data == false) // no data loaded, problem
     {  ws_message('<!-- module '.basename(__FILE__).'('.__LINE__.'): No data retrieved script ends -->',true );
//        echo 'No valid data found<br><br>';
        $AQ_error = 'No valid data found<br><br>';
        $current_AQ     = '?';
//        return false;
		}
// *** let's do some json !
if($AQ_error === false) $json = aq_json_check($new_data);
// success $json example
/*
 Array
(
    [data] => Array
        (
            [did] => 001D0A100021
            [name] => My AirLink
            [ts] => 1599150192
            [conditions] => Array
                (
                    [0] => Array
                        (
                            [lsid] => 123456
                            [data_structure_type] => 6
                            [temp] => 75.8
                            [hum] => 54.3
                            [dew_point] => 58.2
                            [wet_bulb] => 62.7
                            [heat_index] => 76
                            [pm_1_last] => 1
                            [pm_2p5_last] => 1
                            [pm_10_last] => 1
                            [pm_1] => 0.96
                            [pm_2p5] => 1.21
                            [pm_2p5_last_1_hour] => 2.3
                            [pm_2p5_last_3_hours] => 2.29
                            [pm_2p5_last_24_hours] => 4.81
                            [pm_2p5_nowcast] => 2.3
                            [pm_10] => 1.21
                            [pm_10_last_1_hour] => 2.84
                            [pm_10_last_3_hours] => 2.8
                            [pm_10_last_24_hours] => 6.03
                            [pm_10_nowcast] => 2.84
                            [last_report_time] => 1599150192
                            [pct_pm_data_last_1_hour] => 100
                            [pct_pm_data_last_3_hours] => 100
                            [pct_pm_data_nowcast] => 100
                            [pct_pm_data_last_24_hours] => 80
                        )

                )

        )

    [error] => 
)
*/

if($AQ_error === false && $json['error'] != null) {
	ws_message('<!-- module '.basename(__FILE__).'('.__LINE__.'): Sensor problem script ends -->',true );
//	echo "Sensor Problem. Code: ".$json['error']['code']."<br>Message: ".$json['error']['message']."<br><br>";
	$AQ_error = "Sensor Problem. Code: ".$json['error']['code']."<br>Message: ".$json['error']['message']."<br><br>";
    $current_AQ     = '?';
//        return false;
	}

#
// don't do this if error
if($AQ_error === false){

$filetime       = $json['data']['ts'];
$haystack = $json['data']['conditions'][0]; // a 'shortcut' for sensor data


#
# -------------  start beteljuice html stuff
// 'down' triangle
$arrow_black  = '<td> <div style="border-left:10px solid transparent;
border-right:10px solid transparent;
border-top:10px solid #000;
height:0;
width:0;
margin:0 auto;"></div></td>'.PHP_EOL;
// 'up' triangle
$arrow_black2  = '<td> <div style="border-left:10px solid transparent;
border-right:10px solid transparent;
border-bottom:10px solid #000;
height:0;
width:0;
margin:0 auto;"></div></td>'.PHP_EOL;




// betel - construct data table
$nr_data_coloms = 5;    // nr of values in the table (eg. size, class, index, value, period)
$AQ_table        = '<div id="aqhi_data_table" style="display: block;  width: 96%; margin: 10px auto;">
<table class="" style = "width: 100%; border-collapse: collapse;">'.PHP_EOL;
$str_hr         = '<tr style="height: 4px;"><td style="height: 4px;" colspan ="'.($nr_data_coloms+$nr_blocks-1 ).'"><hr /></td></tr>'.PHP_EOL;

// beteljuice - orders changed from original
$AQ_table .= '<tr>
<th style="text-align: center; "><small>'.langtransstr('aq_period')         .'</small></th>
<th style="text-align: center; /* width: 60px; */"><small>'.   langtransstr('aq_particle')       .'</small></th>
<th style="text-align: center /*right*/ ; /* width: 100px; */"><small>'.  langtransstr('aq_concentration')  .'</small></th>
<th style="text-align: center; /* width: 40px; */"><small><b>'.langtransstr('aq_aq')             .'</b></small></th>
<th style="text-align: center;"><small>'.$aqhi_type.'&nbsp;'.langtransstr('aq_classification') .'</small></th>'.PHP_EOL;
$AQ_table         .= str_repeat ('<th>&nbsp;</th>',$nr_blocks-1);
$AQ_table         .= '</tr>'.PHP_EOL;

$current_AQ     = '';

$draw_line = false;
foreach($AQ_time_line as $when) {
	if($when == "1m") {
		$period = langtransstr('aq_last');
		$tl = "";
	} else {
		$period = langtransstr('aq_'.$when);
		if($when == "1h") $tl = "_last_1_hour";
		if($when == "3h") $tl = "_last_3_hours";
		if($when == "12h") $tl = "_nowcast";
		if($when == "24h") $tl = "_last_24_hours";
	}
	if($draw_line) $AQ_table .= $str_hr;

	foreach ($AQ_arr_all as $value) {
		if($value == "1") $pms = "pm_1";
		if($value == "25") $pms = "pm_2p5";
		if($value == "10") $pms = ($haystack['data_structure_type'] == 6 ? "pm_10": "pm_10p0");
		$plug = ($value == "25" && $when == "24h" ? true : false);

		if(array_key_exists($pms.$tl, $haystack)) {
			$draw_line = true;

			$particle= langtransstr('aq_'.strtolower($value));
			$conc   = $haystack[$pms.$tl];		
		
			$measured= $conc.' µg/m<sup>3</sup>';
			list ($ahqi,$key) = calc_aq($conc,$value);
			if($value == 1) list($ahqi,$key) = calc_aq($conc * 1.25, 25); // beteljuice assumpton - 1ug/m normally 80% of 2.5
			$risk   = langtransstr($short_txt.$key);
 // beteljuice - orders changed from original

			$AQ_table    .= '<tr>
<td style="text-align: center;'.($plug ? ' border-left: 1px solid grey; border-top: 1px solid grey; border-bottom: 1px solid grey;' : '').'"><small>'.                $period        .'</small></td>
<td style="text-align: center;'.($plug ? ' border-top: 1px solid grey; border-bottom: 1px solid grey;' : '').' /* width: 60px; */"><small>'.   $particle      .'</small></td>
<td style="text-align: center /*right*/ ;'.($plug ? ' border-top: 1px solid grey; border-bottom: 1px solid grey;' : '').' /* width: 100px; */"><small>'.   $measured      .'</small></td>
<td style="text-align: center;'.($plug ? ' border-top: 1px solid grey; border-bottom: 1px solid grey;' : '').' /* width: 40px; */"><small><b>'.$ahqi          .'</b></small></td>
<td style="text-align: center;'.($plug ? ' border-top: 1px solid grey; border-bottom: 1px solid grey;' : '').'"><small>'.                $risk          .'</small></td>'.PHP_EOL;

			if($plug) {
	
				$width_color = 1+floor(50/$nr_blocks);
				for ($n = 1; $n <= $nr_blocks-1; $n++) {
					$AQ_table .= '<td style = "width: '.$width_color.'%; text-align: center; border: 1px solid grey; color: '
					.$aqhi_color[$n]['color'].';  font-weight:bold; background-color: '
					.$aqhi_color[$n]['bgc'].'; ">'.($key == $n ? '&gt;&nbsp;'.($aqhi_type == "US" ? $ahqi : $n) : '').($key == $n ? '&nbsp;&lt;' : ($aqhi_type == "US" ? '' : $n)).'</td>';
				}
		
			} else {

				$nr24     = $key;
				for ($p = 1; $p <= $nr_blocks - 1; $p++) {
					if ($nr24 == $p) {
//				$string_24h .= $arrow_black;
					$AQ_table .= ($aqhi_type == "US" ? '<td style= "text-align: center;">&gt;&nbsp;'.$ahqi.'&nbsp;&lt;</td>' : $arrow_black);
					} else {
					$AQ_table .= '<td>&nbsp;</td>';
					}
				}
			}
				$AQ_table .= PHP_EOL.'</tr>'.PHP_EOL; 
// ******************

/*			if ($value == $is_current && $when == '1m') 
				{  $current_AQ  = '<span style="'.
                        $current_style.
                        ' color: '.$aqhi_color[$nr24]['color'].'; '.
                        'background-color: '.$aqhi_color[$nr24]['bgc'].';">'.$ahqi.'</span>';}   
*/
		
		} // end array key exists
		
    } // end foreach($AQ_arr_all
	  
} // end foreach($AQ_time_line

$AQ_table        .='</table>'.PHP_EOL; 



// ************ need to do seperate widget check in case they don't want to display 'now' value(s) (beteljuice)

if(!$current_AQ) {
	$conc   = $haystack['pm_2p5_nowcast'];
	list ($ahqi,$key) = calc_aq($conc,'25');
	$current_AQ  = '<span style="'.
                        $current_style.
                        ' color: '.$aqhi_color[$key]['color'].'; '.
                        'background-color: '.$aqhi_color[$key]['bgc'].';">'.$ahqi.'</span>';
	$current_AQ_risk = langtransstr($short_txt.$key);

						}   

# check if arrow is needed  for widget - campare pm2.5 1m with 1h ($arrows_folder is NOT defined ?)
if(!$current_AQ) {
	$conc   = $haystack['pm_2p5_last_1_hour'];
	list ($compare,$key) = calc_aq($conc,'25');
// $compare        = $aq[$is_compare_to]['aqnr'];
	$current        = $ahqi;
	$difference     = $current - $compare;
	$allowed2        = round($is_allowed * $compare /100,1); // beteljuice - var renamed because of conflicts
	$string = '';
	if($difference > $allowed2) 
		{  $string         .= ' higher: '.$difference.' = worse than '.$allowed2.' ('.$is_allowed.'%)';
			$current_AQ     = '<a href="'.$fullAQpage.'">'
                        .'<img style="height: 16px;" src="'.$arrows_folder.'aq_worse.gif" '
                        .' title="'.$string.'" alt =" " /></a>'.$current_AQ;  }
	elseif ($difference < (0 - $allowed2))
		{  $string         .= 'lower: '.$difference.' = better than -'.$allowed2.' ('.$is_allowed.'%)';
		$current_AQ     = '<a href="'.$fullAQpage.'">'.
                        '<img style="height: 16px;" src="'.$arrows_folder.'aq_better.gif" '.
                        ' title="'.$string.'" alt =" " /></a>'.$current_AQ;  }
}// end arrow check
#     

$issued = date($AQ_local_date_fmt,$filetime); 

if (time() > ($filetime  + 1800) ) { $aq_warn_color = 'red'; } else { $aq_warn_color = 'black'; }
#
if($AQtemp_uom == 'C') {
	$AQ_temp = 5* ((float) $haystack['temp'] -32) /9;
	$AQ_temp = round($AQ_temp,1)."&deg;C";
} else {
	$AQ_temp = (float) $haystack['temp']."&deg;F";
}
	

} // end NOT AQ_error


#----------------------------------------------- 
#---------------------------   EXPLANATION BLOCK ($AQ_explain)
#-----------------------------------------------

$coloms         = array ('band','index','P10','P25','risk');
$nr_cols        = count($coloms);
$AQ_explain = '<table style="text-align: center; width: 100%; margin: 0 auto; border-collapse: collapse; border: 1px solid grey;">
<thead style="background-color: #ccc;">
<tr style="border-bottom: 1px solid grey;">'.PHP_EOL;
if (isset ($range_aq) && $range_aq <> false) 
     {  $extra  = 'style="width: 80px;"';}
else {  $extra  = '';}
if (in_array ('band',$coloms) ) 
     {  $AQ_explain .= '<th><small>'.$aqhi_type.' '.langtransstr('aq_classification').'</small></th>'.PHP_EOL;} // #### beteljuice modified
if (in_array ('index',$coloms) )
     {  $AQ_explain .= '<th '.$extra.'><small>'.langtransstr('aq_aq').'</small></th>'.PHP_EOL;}
if (in_array ('P10',$coloms) )
     {  $AQ_explain .= '<th><small>'.langtransstr('aq_10').'</small></th>'.PHP_EOL;}
if (in_array ('P25',$coloms) )
     {  $AQ_explain .= '<th><small>'.langtransstr('aq_25').'</small></th>'.PHP_EOL;}

if (in_array ('risk',$coloms) ) 
     {  $AQ_explain .= '<th><small>'.langtransstr('aq_disclaimer2');
       $AQ_explain .= '</small></th>'.PHP_EOL;}
$AQ_explain .= '</tr></thead>
<tbody>'.PHP_EOL;
#
for ($p = 1; $p <= $nr_blocks - 1; $p++) 
    {   $AQ_explain .= ' 
<tr style="border-bottom: 1px solid grey;">'.PHP_EOL;
        if ($xband_aq <> false )
                if (isset ($xband_aq[$p]))
                     {  $rowspan= $xband_aq[$p]['cnt'];
                        $colornr= $xband_aq[$p]['colors'];
                        $bgcnr  = $xband_aq[$p]['colors'];
                        $textnr = $xband_aq[$p]['text'];
                        $bandprint=true;}
                else {  $bandprint=false;}
        else {  $rowspan= 1;
                $colornr= $p;
                $bgcnr  = $p;
                $textnr = $p;
                $bandprint=true;}
        if (in_array ('band',$coloms) && $bandprint == true ) 
             {  $AQ_explain .=  '<td style="font-weight:bold; '.
                'color: '.              $aqhi_color[$colornr]['color'].'; '.
                'background-color:'.    $aqhi_color[$bgcnr]['bgc'].'; '.'" '.
                'rowspan="'.$rowspan.'">&nbsp;'.
                langtransstr($short_txt.$textnr).'&nbsp;</td>'.PHP_EOL;}

        if (in_array ('index',$coloms) )
             {  if ($xband_aq <> false ) 
                     {  $extra  = 
                        'color:  '.             $aqhi_color[$p]['color'].'; '.
                        'background-color:'.    $aqhi_color[$p]['bgc'].'; ';}
                else {  $extra = '';}
                $AQ_explain .= '<td style="font-weight:bold; border: 1px solid grey; '.
                $extra.'">'.
                $aqhi_range[$p].'</td>'.PHP_EOL;}
        $extra = '';
        if (in_array ('P10',$coloms) )
             {  if ($p < $nr_blocks - 1) 
                     {  $tekst  = '<small>&lt;&nbsp;'.$thrshld_pm10[$p].'&nbsp;µg/m<sup>3</sup></small>';}
                else {  $tekst  = '<small>&gt;&nbsp;'.$thrshld_pm10[$p-1].'&nbsp;µg/m<sup>3</sup></small>';}
                $AQ_explain .= '<td style="border: 1px solid grey; padding: 0px 4px 0px 4px;">'.$tekst.'</td>'.PHP_EOL;} // betel - added padding
        if (in_array ('P25',$coloms) )
             {  if ($p < $nr_blocks - 1) 
                     {  $tekst  = '<small>&lt;&nbsp;'.$thrshld_pm25[$p].'&nbsp;µg/m<sup>3</sup></small>';}
                else {  $tekst  = '<small>&gt;&nbsp;'.$thrshld_pm25[$p-1].'&nbsp;µg/m<sup>3</sup></small>';}
                $AQ_explain .= '<td style="border: 1px solid grey; padding: 0px 4px 0px 4px;">'.$tekst.'</td>'.PHP_EOL;} // betel - added padding
        if (in_array ('risk',$coloms) && $bandprint == true )
             {  $AQ_explain .= '<td rowspan="'.$rowspan.'"><small>'.langtransstr($long_txt.$textnr).'</small></td></tr>'.PHP_EOL;}}

		$AQ_explain .= '<tr style="background-color: #ccc;"><td colspan="'.$nr_cols.'" style="border-top: 1px solid grey;">'.
        langtransstr('aq_visit').
        ' <a href="'.$local_link.'" target="_blank">'.$aqhi_type.' '.langtransstr('aq_link_regional').'</a> '.
        langtransstr('aq_information').'</td></tr>
</tbody>
</table>'.PHP_EOL;
# END $AQ_explain (key table) 

#-----------------------------------------------
#                                      FUNCTIONS
# ----------------------------------------------
#                                 calc_aq
# calculate AQ index based on concentration
# ----------------------------------------------
function calc_aq($conc,$pm = '25')
     {  global $nr_blocks, $thrshld_aq, $thrshld_pm10, $thrshld_pm25;
        $ws_script_environment  = '<!-- module '.basename(__FILE__).' => '.__FUNCTION__;
        if ($pm == '25')
             {  $thrshlds       = $thrshld_pm25;}
        elseif ($pm == '10')
             {  $thrshlds       = $thrshld_pm10;} # echo '<pre>'.__LINE__.print_r($thrshlds,true); exit;
        else {  return array('-','');}
        if ($conc < $thrshlds[0])
             {  return array(-1,0); }        // to low value
        else {  $result         = array(-1,7);          // to high value
                for ($key = 1; $key < $nr_blocks; $key++)
                     {  $value  = $thrshlds[$key];  #echo '<pre>'.__LINE__.' nr = '.$key.' conc = '.$conc.' $value = '.$value;
                        if ($conc < $value) 
                             {  $prev   = (int) $key - 1;
                                $aq_cal = InvLinearPurple( 
                                        $thrshld_aq[$key], $thrshld_aq[$prev],  
                                        $value           , $thrshlds[$prev], 
                                        $conc);
                                $result = array($aq_cal,$key);
                                break;}
                        }}
        ws_message($ws_script_environment.' ('.__LINE__.'): $conc '.$conc.' $pm '.$pm.' $aq_cal '.$aq_cal.' $value ' .$value.' -->',true );
        return $result; } // eof calc_aq
#-----------------------------------------------
# average the two sensor values
#-----------------------------------------------
function ws_average_array($first,$second)
    {   $return = array();
        foreach ($first as $key => $value)
             {  $return[$key] = round( ($first[$key] + $second[$key]) / 2,2);}
        return $return;}
# ----------------------------------------------
#                                InvLinearPurple                                    
# calculation subfunction for calc_aq 
# ----------------------------------------------
function InvLinearPurple($AQIhigh, $AQIlow, $Conchigh, $Conclow, $mes)
     {  $ws_script_environment  = '<!-- module '.basename(__FILE__).' => '.__FUNCTION__;
        $return = ceil((($AQIhigh-$AQIlow) / ($Conchigh-$Conclow) )*($mes-$Conclow) + $AQIlow);
        ws_message($ws_script_environment.' ('.__LINE__.'): $return '.$return.' $AQIhigh '.$AQIhigh.' $AQIlow '.$AQIlow.' $Conchigh '.$Conchigh.' $Conclow '.$Conclow.' $mes '.$mes.' -->',true);
        return $return;}
# ----------------------------------------------
# arrays 
# ----------------------------------------------        
function aqhi_load_arrays() 
     {  global  $aqhi_type, $aqhi_range, $aqhi_color, $local_link, $range_aq,
                $nr_blocks, $thrshld_aq, $thrshld_pm10, $thrshld_pm25, $long_txt , $short_txt, $xband_aq;   
        if ($aqhi_type == 'BE')
             {  $local_link     = 'http://www.irceline.be/nl/luchtkwaliteit/metingen/belaqi-luchtkwaliteitsindex/informatie';
                $thrshld_aq     = array (0,  1,  2,  3,  4,  5,  6,  7,  8,  9,   10, 99999);
                $thrshld_pm10   = array (0, 10, 20, 30, 40, 50, 60, 70, 80, 100, 500, 99999);
                $thrshld_pm25   = array (0,  5, 10, 15, 25, 35, 40, 50, 60,  70, 500, 99999);
                $nr_blocks      = count($thrshld_aq) - 1;
                $long_txt       = 'aq_txt_lng_be';
                $short_txt      = 'aq_txt_sh_be';
                $xband_aq        = array ();
                $xband_aq[1]     = array ('text' => 1, 'cnt' => 2,  'colors' => 1 );
                $xband_aq[3]     = array ('text' => 3, 'cnt' => 2,  'colors' => 3 );
                $xband_aq[5]     = array ('text' => 6, 'cnt' => 3,  'colors' => 6 );
                $xband_aq[8]     = array ('text' => 9, 'cnt' => 3,  'colors' => 9);
                $aqhi_color[0]   = array ('bgc' => 'rgb(0,0,0)'      , 'color' => '#000');
                $aqhi_color[1]   = array ('bgc' => 'rgb(3,46,235)'   , 'color' => '#FFF');
                $aqhi_color[2]   = array ('bgc' => 'rgb(90,153,2407)', 'color' => '#FFF');
                $aqhi_color[3]   = array ('bgc' => 'rgb(90,146,59)'  , 'color' => '#000'); #160,205,98
                $aqhi_color[4]   = array ('bgc' => 'rgb(155,246,104)', 'color' => '#000');
                $aqhi_color[5]   = array ('bgc' => 'rgb(255,252,114)', 'color' => '#000');
                $aqhi_color[6]   = array ('bgc' => 'rgb(236,189,89)' , 'color' => '#000');
                $aqhi_color[7]   = array ('bgc' => 'rgb(220,118,64)' , 'color' => '#000');
                $aqhi_color[8]   = array ('bgc' => 'rgb(215,69,51)'  , 'color' => '#FFF');
                $aqhi_color[9]   = array ('bgc' => 'rgb(128,37,25)'  , 'color' => '#FFF'); 
                $aqhi_color[10]  = array ('bgc' => 'rgb(84,22,13)'   , 'color' => '#FFF');
                $aqhi_color[11]  = array ('bgc' => 'rgb(0,0,0)'      , 'color' => '#FFF'); } 
        elseif ($aqhi_type == 'IE')
             {  $local_link     = 'http://www.epa.ie/air/quality/';
                $thrshld_aq     = array (0,  1,  2,  3,  4,  5,  6,  7,  8,  9,   10, 99999);
                $thrshld_pm10   = array (0, 16, 33, 50, 58, 66, 75, 83, 91, 100, 500, 99999);
                $thrshld_pm25   = array (0, 11, 23, 35, 41, 47, 53, 58, 64,  70, 500, 99999);
                $nr_blocks      = count($thrshld_aq) - 1;
                $long_txt       = 'aq_txt_lng_ie';
                $short_txt      = 'aq_txt_sh_ie';
                $xband_aq        = array ();
                $xband_aq[1]     = array ('text' => 2, 'cnt' => 3,  'colors' => 2 );
                $xband_aq[4]     = array ('text' => 5, 'cnt' => 3,  'colors' => 5 );
                $xband_aq[7]     = array ('text' => 8, 'cnt' => 3,  'colors' => 8 );
                $xband_aq[10]    = array ('text' => 10,'cnt' => 1,  'colors' => 10);
                $aqhi_color[0]   = array ('bgc' => 'rgb(0,0,0)'      , 'color' => '#000');
                $aqhi_color[1]   = array ('bgc' => 'rgb(196,213,82)' , 'color' => '#000');
                $aqhi_color[2]   = array ('bgc' => 'rgb(121,176,84)' , 'color' => '#000');
                $aqhi_color[3]   = array ('bgc' => 'rgb(75,129,61)'  , 'color' => '#000');;
                $aqhi_color[4]   = array ('bgc' => 'rgb(233,191,70)' , 'color' => '#000');
                $aqhi_color[5]   = array ('bgc' => 'rgb(232,151,63)' , 'color' => '#000');
                $aqhi_color[6]   = array ('bgc' => 'rgb(224,109,56)' , 'color' => '#000');
                $aqhi_color[7]   = array ('bgc' => 'rgb(224,109,56)' , 'color' => '#000');
                $aqhi_color[8]   = array ('bgc' => 'rgb(216,56,50)'  , 'color' => '#000');
                $aqhi_color[9]   = array ('bgc' => 'rgb(161,39,34)'  , 'color' => '#000');
                $aqhi_color[10]  = array ('bgc' => 'rgb(108,57,31)'  , 'color' => '#000');
                $aqhi_color[11]  = array ('bgc' => 'rgb(165,73,147)' , 'color' => '#000'); }
// UK threshold values same as IE but colours slightly different - plus info link
        elseif ($aqhi_type == 'UK')
             {  $local_link     = 'https://uk-air.defra.gov.uk/air-pollution/daqi?view=more-info&pollutant=pm25#pollutant';
                $thrshld_aq     = array (0,  1,  2,  3,  4,  5,  6,  7,  8,  9,   10, 99999);
                $thrshld_pm10   = array (0, 16, 33, 50, 58, 66, 75, 83, 91, 100, 500, 99999);
                $thrshld_pm25   = array (0, 11, 23, 35, 41, 47, 53, 58, 64,  70, 500, 99999);
                $nr_blocks      = count($thrshld_aq) - 1;
                $long_txt       = 'aq_txt_lng_ie'; // using ie descriptions
                $short_txt      = 'aq_txt_sh_ie'; // using ie descriptions
                $xband_aq        = array ();
                $xband_aq[1]     = array ('text' => 2, 'cnt' => 3,  'colors' => 2 );
                $xband_aq[4]     = array ('text' => 5, 'cnt' => 3,  'colors' => 5 );
                $xband_aq[7]     = array ('text' => 8, 'cnt' => 3,  'colors' => 8 );
                $xband_aq[10]    = array ('text' => 10,'cnt' => 1,  'colors' => 10);
                $aqhi_color[0]   = array ('bgc' => 'rgb(0,0,0)'      , 'color' => '#000');
                $aqhi_color[1]   = array ('bgc' => '#9cff9c' , 'color' => '#000');
                $aqhi_color[2]   = array ('bgc' => '#31ff00' , 'color' => '#000');
                $aqhi_color[3]   = array ('bgc' => '#31cf00'  , 'color' => '#000');;
                $aqhi_color[4]   = array ('bgc' => '#ff0' , 'color' => '#000');
                $aqhi_color[5]   = array ('bgc' => '#ffcf00' , 'color' => '#000');
                $aqhi_color[6]   = array ('bgc' => '#ff9a00' , 'color' => '#000');
                $aqhi_color[7]   = array ('bgc' => '#ff6464' , 'color' => '#000');
                $aqhi_color[8]   = array ('bgc' => '#f00'  , 'color' => '#fff');
                $aqhi_color[9]   = array ('bgc' => '#900'  , 'color' => '#fff');
                $aqhi_color[10]  = array ('bgc' => '#ce30ff'  , 'color' => '#fff');
                $aqhi_color[11]  = array ('bgc' => 'rgb(165,73,147)' , 'color' => '#000'); } // why extra entry ?
        elseif ($aqhi_type == 'NL')
             {  $local_link     = 'https://www.luchtmeetnet.nl/stations/alle-provincies/alle-gemeentes/PM25';
                $thrshld_aq     = array (0,  1,  2,  3,  4,  5,  6,   7,   8,   9,  10, 11, 99999);
                $thrshld_pm10   = array (0, 10, 20, 30, 45, 60, 75, 100, 125, 150, 200, 99999);
                $thrshld_pm25   = array (0, 10, 15, 20, 30, 40, 50,  70,  90, 100, 140, 99999);
                $nr_blocks      = count($thrshld_aq) - 1;
                $long_txt       = 'aq_txt_lng_nl';
                $short_txt      = 'aq_txt_sh_nl';
                $xband_aq        = array ();
                $xband_aq[1]     = array ('text' => 2, 'cnt' => 3,  'colors' => 2 );
                $xband_aq[4]     = array ('text' => 5, 'cnt' => 3,  'colors' => 5 );
                $xband_aq[7]     = array ('text' => 7, 'cnt' => 2,  'colors' => 7 );
                $xband_aq[9]     = array ('text' => 9, 'cnt' => 2,  'colors' => 9 );
                $xband_aq[11]    = array ('text' => 11,'cnt' => 1,  'colors' => 11 );
                $aqhi_color[0]   = array ('bgc' => 'rgb(0,0,0)'      , 'color' => '#000');
                $aqhi_color[1]   = array ('bgc' => 'rgb(0,29,245)'   , 'color' => '#FFF');
                $aqhi_color[2]   = array ('bgc' => 'rgb(68,152,247)' , 'color' => '#FFF');
                $aqhi_color[3]   = array ('bgc' => 'rgb(154,186,240)', 'color' => '#000');;
                $aqhi_color[4]   = array ('bgc' => 'rgb(244,212,119)' , 'color' => '#000');
                $aqhi_color[5]   = array ('bgc' => 'rgb(255,254,84)'  , 'color' => '#000');
                $aqhi_color[6]   = array ('bgc' => 'rgb(255,252,172)' , 'color' => '#000');
                $aqhi_color[7]   = array ('bgc' => 'rgb(238,136,80)'  , 'color' => '#000');
                $aqhi_color[8]   = array ('bgc' => 'rgb(244,189,65)'  , 'color' => '#FFF');
                $aqhi_color[9]   = array ('bgc' => 'rgb(236,11,46)'   , 'color' => '#FFF');
                $aqhi_color[10]  = array ('bgc' => 'rgb(233,51,36)'   , 'color' => '#FFF');
                $aqhi_color[11]  = array ('bgc' => 'rgb(102,55,153)'  , 'color' => '#FFF');                
                $aqhi_color[12]  = array ('bgc' => 'rgb(0,0,0)'       , 'color' => '#FFF'); } 
        elseif ($aqhi_type == 'eea')
			 {  $local_link     = 'https://www.eea.europa.eu/themes/air/air-quality-index'; // experiment, generic EU class
                $thrshld_aq     = array (0,   1,  2,  3,   4,   5,   6,   99999);
                $thrshld_pm10   = array (0,   20,  40,  50, 100, 150, 1200,   99999);
                $thrshld_pm25   = array (0.0, 10, 20 , 25, 50, 75, 800, 99999);
                $nr_blocks      = count($thrshld_aq) - 1;
                $long_txt       = 'aq_txt_lng_eea';
                $short_txt      = 'aq_txt_sh_eea';
                $xband_aq       = false;
                $range_aq       = true;
                $aqhi_color[0]    = array ('bgc' => 'rgb(0,0,0)'     , 'color' => '#000');
                $aqhi_color[1]    = array ('bgc' => 'rgba(80,240,230, 1)'   , 'color' => '#000');
                $aqhi_color[2]    = array ('bgc' => 'rgba(80,204,170, 1)' , 'color' => '#000');
                $aqhi_color[3]    = array ('bgc' => 'rgba(240,230,65, 1)' , 'color' => '#000');;
                $aqhi_color[4]    = array ('bgc' => 'rgba(255,80,80, 1)'   , 'color' => '#FFF');
                $aqhi_color[5]    = array ('bgc' => 'rgba(150,0,50, 1)'  , 'color' => '#FFF');
                $aqhi_color[6]    = array ('bgc' => 'rgba(125, 33, 129, 1)'   , 'color' => '#FFF');
                $aqhi_color[7]    = array ('bgc' => 'rgb(0,0,0)'     , 'color' => '#FFF'); }
        else {  $local_link     = 'https://www.airnow.gov/aqi/aqi-basics/'; // DEFAULT US
                $thrshld_aq     = array (0,   50,  100,  150,   200,   300,   500,   99999);
                $thrshld_pm10   = array (0,   54,  154,  254,   354,   424,   604,   99999);
                $thrshld_pm25   = array (0.0, 12.0, 35.4 ,55.4, 150.4, 250.4, 500.4, 99999);
                $nr_blocks      = count($thrshld_aq) - 1;
                $long_txt       = 'aq_txt_lng_us';
                $short_txt      = 'aq_txt_sh_us';
                $xband_aq       = false;
                $range_aq       = true;
                $aqhi_color[]    = array ('bgc' => 'rgb(0,0,0)'     , 'color' => '#000');
                $aqhi_color[]    = array ('bgc' => 'rgb(0,228,0)'   , 'color' => '#000');
                $aqhi_color[]    = array ('bgc' => 'rgb(255,255,0)' , 'color' => '#000');
                $aqhi_color[]    = array ('bgc' => 'rgb(255,126,0)' , 'color' => '#000');;
                $aqhi_color[]    = array ('bgc' => 'rgb(255,0,0)'   , 'color' => '#FFF');
                $aqhi_color[]    = array ('bgc' => 'rgb(153,0,76)'  , 'color' => '#FFF');
                $aqhi_color[]    = array ('bgc' => 'rgb(76,0,38)'   , 'color' => '#FFF');
                $aqhi_color[]    = array ('bgc' => 'rgb(0,0,0)'     , 'color' => '#FFF'); }
        $previous       = '';
        $aqhi_range      = array(0);
        for ($n = 0; $n <= $nr_blocks; $n++) 
             {  if ($n == $nr_blocks)   // mostly invalid to large value
                     {  $aqhi_range[$n]  = '>'.$thrshld_aq[$n];}
                elseif ($n == 0)        // mostly invalid < 0
                     {  $aqhi_range[$n]  = '<'.       $thrshld_aq[$n];}
                else {  if (($thrshld_aq[$n - 1] + 1) == $thrshld_aq[$n]) {$previous = '';}
                        $aqhi_range[$n]  = $previous. $thrshld_aq[$n];}
                $previous                       = ($thrshld_aq[$n]+1).'-';}
}//  eof aqhi_load_arrays

#-----------------------------------------------
# load all "missing" template functions
# ----------------------------------------------    
function load_missing() {
    if (!function_exists ('ws_message') ){
        function ws_message ($message,$always=false,&$string=false) 
             {  global $wsDebug, $SITE;
                $echo	= $always;
                if ( $echo == false && isset ($wsDebug) && $wsDebug == true ) 			{$echo = true;}
                if ( $echo == false && isset ($SIE['wsDebug']) && $SIE['wsDebug'] == true ) 	{$echo = true;}
                if ( $echo == true  && $string === false) {echo $message.PHP_EOL;}
                if ( $echo == true  && $string <> false) {$string .= $message.PHP_EOL;}}
    } // check ws_message

    if (!function_exists ('ws_makeRequest') ){
        function ws_makeRequest($file, $get_contents = false, $fake= '') 
              { $ws_script_environment = '<!-- module '.basename(__FILE__).' => '.__FUNCTION__;
                if ($fake == '') {$fake = $file;}
                if ($get_contents === false || $get_contents == '')   // use CURL to load http(s) data
                     {  ws_message($ws_script_environment.' ('.__LINE__.'): data will be loaded from '.$fake.' -->',true ); 
                        return ws_curl($file,  $fake);}
                if ($get_contents === true)    // use file_get_contents to load http(s) data
                     {  $file_to_load   = $file; } 
                else {  $file_to_load   = $get_contents;} // use file_get_contents to load local (or test) file
                ws_message($ws_script_environment.' ('.__LINE__.'): data will be loaded from file '.$file_to_load.' using file get contents -->',true );
                return file_get_contents($file_to_load);} // eof ws_makeRequest
    } // eo check ws_makeRequest
    if (!function_exists ('ws_curl') ){
        function ws_curl ($url = '',$fake = '') {
                global $ws_msg_string, $ws_curl_follow, $ws_timeout, $ws_user_agent ;
                $ws_script_environment = '<!-- module '.basename(__FILE__).' => '.__FUNCTION__;
                if ($fake == '') {$fake = $url;}
                if (!isset ($ws_timeout) || $ws_timeout < 10)   { $ws_timeout   = 10;}
                if (!isset ($ws_user_agent) )                   { $ws_user_agent= $_SERVER['HTTP_USER_AGENT'];}
                ws_message(ws_debug_times('module '.basename(__FILE__).' => '.__FUNCTION__.' ('.__LINE__.'):'),true);
                ws_message($ws_script_environment.' ('.__LINE__.'): data loaded from '.$url.' -->',false , $ws_msg_string );		
                $ch     = curl_init();                                  // initialize a cURL session
                curl_setopt ($ch, CURLOPT_URL, $url);                   // connect to provided URL
                curl_setopt ($ch, CURLOPT_SSL_VERIFYPEER, 0);           // verify peer off,removes a lot of errors with older hosts
                curl_setopt ($ch, CURLOPT_USERAGENT, $ws_user_agent);   // most host checks this nowadays
                curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, $ws_timeout); // connection timeout
                curl_setopt ($ch, CURLOPT_TIMEOUT, $ws_timeout);        // data timeout
                curl_setopt ($ch, CURLOPT_RETURNTRANSFER, true);        // return the data transfer
                curl_setopt ($ch, CURLOPT_NOBODY, false);               // do the download request without getting the body
                curl_setopt ($ch, CURLOPT_HEADER, false);               // include header information
                if (! isset ($ws_curl_follow) ) {$ws_curl_follow = 1;}
                curl_setopt($ch, CURLOPT_FOLLOWLOCATION, $ws_curl_follow); // follow Location: redirect allowed
                curl_setopt($ch, CURLOPT_MAXREDIRS, 1);                 //   but only one time
                $rawData= curl_exec ($ch);
                ws_message(ws_debug_times('module '.basename(__FILE__).' => '.__FUNCTION__.' ('.__LINE__.'):'),true);
                $info	= curl_getinfo($ch);
                $errors = curl_error($ch);
                curl_close ($ch);
                unset ($ch);
                ws_message($ws_script_environment.' ('.__LINE__.'): Return codes:'.print_r ($info,true).' -->');
                ws_message($ws_script_environment.' ('.__LINE__.'): Error  codes:'.print_r ($errors,true).' -->');
                if (strlen($rawData) > 2) { return $rawData;}
        #        echo '<p><b>No data</b> was retrieved for '.$url.'</p>'.PHP_EOL;
                return false;}  // eo curl function 
    } // eo check ws_curl
    if (!function_exists ('ws_debug_times') ){
        function ws_debug_times($start = '') {
                global $ws_start_time, $ws_passed_time;
                if (!isset ($ws_passed_time)) 
                     {  $ws_start_time = $ws_passed_time = microtime(true);
                        return  '<!-- '.$start.' Debug timers initialized  -->';}
        #
                $now            = microtime(true);
                $since_last     = $now - $ws_passed_time;
                $ws_passed_time = $now;
        
                if ($since_last < 0.0001) {$string1 = '< 0.0001';} else {$string1 = round($since_last,4);}
                $until_last     = $now - $ws_start_time;
                if ($until_last < 0.0001) {$string2 = '< 0.0001';} else {$string2 = round($until_last,4);}
       
                return '<!-- '.$start.' Time spent until here: '.$string2.' seconds. Since last timestamp: '.$string1.' seconds.  -->';
        } // eof ws_debug_times
    }  // eo exists ws_debug_times?
    if (!function_exists ('ws_debug_info') ){
        function ws_debug_info($start = '') {
                global $wsLANGMISSING, $ws_start_time, $ws_passed_time, $ws_lang_dir, $lang;
                if (isset ($wsLANGMISSING) && count($wsLANGMISSING) > 0) {
                        $string = '# missing entries added at '.date('c').PHP_EOL;
                        foreach ($wsLANGMISSING as  $value => $script)
                        # 2017-12-03 add script name for missing translation
                             {  $string .= 'langlookup|'.$value.'|'.$value.'|'.$script.PHP_EOL;}
                        $langfile 	= $ws_lang_dir. 'fctlanguage-'.$lang.'.txt';
                        $return = file_put_contents($langfile,$string,FILE_APPEND);
                        $return = PHP_EOL.'<!-- '. $string.PHP_EOL.' -->'.PHP_EOL;
                } // eo lang missing
                else  { $return = '';}
        #
                $return .= PHP_EOL.ws_debug_times($start);
        #
                $size   = memory_get_peak_usage(true);
                $unit   = array('b','kb','mb','gb','tb','pb');
                $used   = round($size/pow(1024,($i=floor(log($size,1024)))),2).' '.$unit[$i];
                $seconds= microtime(true) - $ws_start_time;
                return  $return.PHP_EOL.'<!-- Max memory used: '.$used.'('.$size.' bytes). Duration: '.round($seconds,5).' seconds -->'.PHP_EOL;
        }
    } // eo exists ws_debug_info
    if (!function_exists ('aq_json_check') ){
#--------------------------------------------------------------------------------------------------
# ws_noaa_json_check check if there are errors in the json,  thanks to Ken True for this code snippet	
#--------------------------------------------------------------------------------------------------
        function aq_json_check ($rawData = '',$check_contents = false) { 
			global $AQ_error;
			if (strlen($rawData) < 10 )             {return false;}
                $array  = json_decode($rawData,true);           // parse the JSON into an associative array
                if (!function_exists('json_last_error')){return $array;}
                $error  = true;
                switch (json_last_error()) 
                     {  case JSON_ERROR_NONE:           $JSONerror = '- No errors';  $error = false;            break;
                        case JSON_ERROR_DEPTH:          $JSONerror = '- Maximum stack depth exceeded';          break;
                        case JSON_ERROR_STATE_MISMATCH: $JSONerror = '- Underflow or the modes mismatch';       break;
                        case JSON_ERROR_CTRL_CHAR:      $JSONerror = '- Unexpected control character found';    break;
                        case JSON_ERROR_SYNTAX:         $JSONerror = '- Syntax error, malformed JSON';          break;
                        case JSON_ERROR_UTF8:           $JSONerror = '- Malformed UTF-8 characters, possibly incorrectly encoded'; break;
                        default:                        $JSONerror = '- Unknown error';                         break;  }
                if ($error) 
                     {  ws_message ('<!-- module '.basename(__FILE__).' => '.__FUNCTION__.' => '.'('.__LINE__.'): error = '.$JSONerror.' -->',true);
                        ws_message ('<!-- content='.print_r($rawData,true).' -->',true);
						$AQ_error = 'JSON file error = '.$JSONerror.'<br><br>';
                        return false;}
                return $array;} // eof aq_json_check
    } // eo exists aq_json_check
 
} // eof load_missing
#-----------------------------------------------
# load language translations and optional function
# ----------------------------------------------    
function ws_load_aq_langs ()
 {      global $lang, $LANGLOOKUP, $aqhi_type;
        $ws_script_environment  = '<!-- module '.basename(__FILE__).' => '.__FUNCTION__;
        if (!isset ($lang)) {$lang = 'en';}
        if (!function_exists('langtransstr'))
             {  function langtransstr ($string) 
                     {  global $LANGLOOKUP;
                        if (isset ($LANGLOOKUP[$string])) 
                             {  return $LANGLOOKUP[$string]; }
                        else {return $string;} }  }      
#
$AQ_lang_file      = __DIR__.'/ws_AQ_lang'.strtoupper($lang).'.php';
if (is_file ($AQ_lang_file)) 
     {  ws_message($ws_script_environment.' ('.__LINE__.'): loading '.$AQ_lang_file.' -->',true);
        include $AQ_lang_file;}
else {  ws_message($ws_script_environment.' ('.__LINE__.'): '.$AQ_lang_file.' not found -->',true);
        $AQ_lang_file      = __DIR__.'/ws_langEN.php';
        if (is_file ($AQ_lang_file)) 
             {  ws_message($ws_script_environment.' ('.__LINE__.'): loading '.$AQ_lang_file.' -->',true);
                include $AQ_lang_file;}
        ws_message($ws_script_environment.' ('.__LINE__.'): '.$AQ_lang_file.' not found -->',true);}
} // eof ws_load_aq_langs
